@@ -51,7 +51,7 @@ namespace MagickCore
 // Borland C++Builder and MinGW builds.
 //
 #if defined(WIN32) && !defined(__CYGWIN__) && !defined(__MINGW32__)
-# define MagickCplusPlusDLLSupported
+#  define MagickCplusPlusDLLSupported
 #endif
 #if defined(MagickCplusPlusDLLSupported)
 #  if defined(_MT) && defined(_DLL) && !defined(_LIB) && !defined(STATIC_MAGICK)
@@ -74,39 +74,27 @@ namespace MagickCore
 #    endif
 #    if !defined(MAGICKCORE_IMPLEMENTATION)
 #      if defined(__GNUC__)
-#       define MagickPPExport __attribute__ ((dllimport))
+#        define MagickPPExport __attribute__ ((dllimport))
 #      else
-#       define MagickPPExport __declspec(dllimport)
+#        define MagickPPExport __declspec(dllimport)
 #      endif
 #      define MagickPPPrivate extern __declspec(dllimport)
-#      if defined(_VISUALC_)
-#        pragma message( "Magick++ lib DLL import" )
-#      endif
 #    else
 #      if defined(__BORLANDC__) || defined(__MINGW32__)
 #        define MagickPPExport __declspec(dllexport)
 #        define MagickPPPrivate __declspec(dllexport)
-#        if defined(__BORLANDC__)
-#          pragma message( "BCBMagick++ lib DLL export" )
-#        endif
 #      else
 #        if defined(__GNUC__)
-#         define MagickPPExport __attribute__ ((dllexport))
+#          define MagickPPExport __attribute__ ((dllexport))
 #        else
-#         define MagickPPExport __declspec(dllexport)
+#          define MagickPPExport __declspec(dllexport)
 #        endif
 #        define MagickPPPrivate extern __declspec(dllexport)
 #      endif
-#      if defined(_VISUALC_)
-#        pragma message( "Magick++ lib DLL export" )
-#      endif
 #    endif
 #  else
 #    define MagickPPExport
 #    define MagickPPPrivate
-#    if defined(_VISUALC_)
-#      pragma message( "Magick++ lib static interface" )
-#    endif
 #    if defined(_MSC_VER) && defined(STATIC_MAGICK) && !defined(NOAUTOLINK_MAGICK)
 #      if defined(_DEBUG)
 #        if defined(MAGICKCORE_BZLIB_DELEGATE)
@@ -266,13 +254,13 @@ namespace MagickCore
 #    endif
 #  endif
 #else
-# if __GNUC__ >= 4
-#  define MagickPPExport __attribute__ ((visibility ("default")))
-#  define MagickPPPrivate  __attribute__ ((visibility ("hidden")))
-# else
-#   define MagickPPExport
-#   define MagickPPPrivate
-# endif
+#  if __GNUC__ >= 4
+#    define MagickPPExport __attribute__ ((visibility ("default")))
+#    define MagickPPPrivate  __attribute__ ((visibility ("hidden")))
+#  else
+#    define MagickPPExport
+#    define MagickPPPrivate
+#  endif
 #endif
 
 #if (defined(WIN32) || defined(WIN64)) && defined(_VISUALC_)
@@ -466,26 +454,28 @@ namespace Magick
   using MagickCore::CompressionType;
   using MagickCore::UndefinedCompression;
   using MagickCore::NoCompression;
+  using MagickCore::B44ACompression;
+  using MagickCore::B44Compression;
   using MagickCore::BZipCompression;
   using MagickCore::DXT1Compression;
   using MagickCore::DXT3Compression;
   using MagickCore::DXT5Compression;
   using MagickCore::FaxCompression;
   using MagickCore::Group4Compression;
-  using MagickCore::JPEGCompression;
-  using MagickCore::JPEG2000Compression;
-  using MagickCore::LosslessJPEGCompression;
-  using MagickCore::LZWCompression;
-  using MagickCore::RLECompression;
-  using MagickCore::ZipCompression;
-  using MagickCore::ZipSCompression;
-  using MagickCore::PizCompression;
-  using MagickCore::Pxr24Compression;
-  using MagickCore::B44Compression;
-  using MagickCore::B44ACompression;
-  using MagickCore::LZMACompression;
   using MagickCore::JBIG1Compression;
   using MagickCore::JBIG2Compression;
+  using MagickCore::JPEG2000Compression;
+  using MagickCore::JPEGCompression;
+  using MagickCore::LosslessJPEGCompression;
+  using MagickCore::LZMACompression;
+  using MagickCore::LZWCompression;
+  using MagickCore::PizCompression;
+  using MagickCore::Pxr24Compression;
+  using MagickCore::RLECompression;
+  using MagickCore::WebPCompression;
+  using MagickCore::ZipCompression;
+  using MagickCore::ZipSCompression;
+  using MagickCore::ZstdCompression;
 
   // Decoration types
   using MagickCore::DecorationType;
