@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -138,9 +138,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
     }
   draw_info->text=ConstantString(label);
-  metrics.width=0.0;
-  metrics.height=0.0;
-  metrics.ascent=0.0;
+  (void) memset(&metrics,0,sizeof(metrics));
   status=GetMultilineTypeMetrics(image,draw_info,&metrics);
   if ((image->columns == 0) && (image->rows == 0))
     {
