@@ -23,13 +23,13 @@
 %                               February 2000                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -156,9 +156,9 @@ MagickExport CacheView *AcquireVirtualCacheView(const Image *image,
   CacheView
     *magick_restrict cache_view;
 
-  magick_unreferenced(image);
+  magick_unreferenced(exception);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
 #if defined(MAGICKCORE_OPENCL_SUPPORT)
@@ -168,7 +168,7 @@ MagickExport CacheView *AcquireVirtualCacheView(const Image *image,
     sizeof(*cache_view)));
   if (cache_view == (CacheView *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(cache_view,0,sizeof(*cache_view));
+  (void) memset(cache_view,0,sizeof(*cache_view));
   cache_view->image=ReferenceImage((Image *) image);
   cache_view->number_threads=GetOpenMPMaximumThreads();
   if (GetMagickResourceLimit(ThreadResource) > cache_view->number_threads)
@@ -178,7 +178,7 @@ MagickExport CacheView *AcquireVirtualCacheView(const Image *image,
   cache_view->nexus_info=AcquirePixelCacheNexus(cache_view->number_threads);
   cache_view->virtual_pixel_method=GetImageVirtualPixelMethod(image);
   cache_view->debug=IsEventLogging();
-  cache_view->signature=MagickSignature;
+  cache_view->signature=MagickCoreSignature;
   if (cache_view->nexus_info == (NexusInfo **) NULL)
     ThrowFatalException(CacheFatalError,"UnableToAcquireCacheView");
   return(cache_view);
@@ -212,7 +212,7 @@ MagickExport CacheView *CloneCacheView(const CacheView *cache_view)
     *magick_restrict clone_view;
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -220,13 +220,13 @@ MagickExport CacheView *CloneCacheView(const CacheView *cache_view)
     sizeof(*clone_view)));
   if (clone_view == (CacheView *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(clone_view,0,sizeof(*clone_view));
+  (void) memset(clone_view,0,sizeof(*clone_view));
   clone_view->image=ReferenceImage(cache_view->image);
   clone_view->number_threads=cache_view->number_threads;
   clone_view->nexus_info=AcquirePixelCacheNexus(cache_view->number_threads);
   clone_view->virtual_pixel_method=cache_view->virtual_pixel_method;
   clone_view->debug=cache_view->debug;
-  clone_view->signature=MagickSignature;
+  clone_view->signature=MagickCoreSignature;
   return(clone_view);
 }
 
@@ -256,7 +256,7 @@ MagickExport CacheView *CloneCacheView(const CacheView *cache_view)
 MagickExport CacheView *DestroyCacheView(CacheView *cache_view)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -264,7 +264,7 @@ MagickExport CacheView *DestroyCacheView(CacheView *cache_view)
     cache_view->nexus_info=DestroyPixelCacheNexus(cache_view->nexus_info,
       cache_view->number_threads);
   cache_view->image=DestroyImage(cache_view->image);
-  cache_view->signature=(~MagickSignature);
+  cache_view->signature=(~MagickCoreSignature);
   cache_view=(CacheView *) RelinquishAlignedMemory(cache_view);
   return(cache_view);
 }
@@ -295,7 +295,7 @@ MagickExport CacheView *DestroyCacheView(CacheView *cache_view)
 MagickExport size_t GetCacheViewChannels(const CacheView *cache_view)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -328,7 +328,7 @@ MagickExport size_t GetCacheViewChannels(const CacheView *cache_view)
 MagickExport ColorspaceType GetCacheViewColorspace(const CacheView *cache_view)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -361,7 +361,7 @@ MagickExport ColorspaceType GetCacheViewColorspace(const CacheView *cache_view)
 MagickExport ExceptionInfo *GetCacheViewException(const CacheView *cache_view)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -398,7 +398,7 @@ MagickExport MagickSizeType GetCacheViewExtent(const CacheView *cache_view)
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -434,7 +434,7 @@ MagickExport MagickSizeType GetCacheViewExtent(const CacheView *cache_view)
 MagickExport ClassType GetCacheViewStorageClass(const CacheView *cache_view)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -478,7 +478,7 @@ MagickExport PixelPacket *GetCacheViewAuthenticPixels(CacheView *cache_view,
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(id < (int) cache_view->number_threads);
   return(GetAuthenticPixelCacheNexus(cache_view->image,x,y,columns,rows,
     cache_view->nexus_info[id],exception));
@@ -526,7 +526,7 @@ MagickExport MagickBooleanType GetOneCacheViewAuthenticPixel(
     *magick_restrict pixels;
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   *pixel=cache_view->image->background_color;
   assert(id < (int) cache_view->number_threads);
   pixels=GetAuthenticPixelCacheNexus(cache_view->image,x,y,1,1,
@@ -567,7 +567,7 @@ MagickExport IndexPacket *GetCacheViewAuthenticIndexQueue(CacheView *cache_view)
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(cache_view->image->cache != (Cache) NULL);
   assert(id < (int) cache_view->number_threads);
   return(cache_view->nexus_info[id]->indexes);
@@ -604,7 +604,7 @@ MagickExport PixelPacket *GetCacheViewAuthenticPixelQueue(CacheView *cache_view)
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(cache_view->image->cache != (Cache) NULL);
   assert(id < (int) cache_view->number_threads);
   return(cache_view->nexus_info[id]->pixels);
@@ -642,7 +642,7 @@ MagickExport const IndexPacket *GetCacheViewVirtualIndexQueue(
     id = GetOpenMPThreadId();
 
   assert(cache_view != (const CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(cache_view->image->cache != (Cache) NULL);
   assert(id < (int) cache_view->number_threads);
   return(GetVirtualIndexesFromNexus(cache_view->image->cache,
@@ -681,7 +681,7 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixelQueue(
     id = GetOpenMPThreadId();
 
   assert(cache_view != (const CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(cache_view->image->cache != (Cache) NULL);
   assert(id < (int) cache_view->number_threads);
   return(GetVirtualPixelsNexus(cache_view->image->cache,
@@ -728,9 +728,9 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixels(
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(id < (int) cache_view->number_threads);
-  return(GetVirtualPixelsFromNexus(cache_view->image,
+  return(GetVirtualPixelCacheNexus(cache_view->image,
     cache_view->virtual_pixel_method,x,y,columns,rows,
     cache_view->nexus_info[id],exception));
 }
@@ -778,10 +778,10 @@ MagickExport MagickBooleanType GetOneCacheViewVirtualPixel(
     *magick_restrict pixels;
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   *pixel=cache_view->image->background_color;
   assert(id < (int) cache_view->number_threads);
-  pixels=GetVirtualPixelsFromNexus(cache_view->image,
+  pixels=GetVirtualPixelCacheNexus(cache_view->image,
     cache_view->virtual_pixel_method,x,y,1,1,cache_view->nexus_info[id],
     exception);
   if (pixels == (const PixelPacket *) NULL)
@@ -837,10 +837,10 @@ MagickExport MagickBooleanType GetOneCacheViewVirtualMethodPixel(
     *magick_restrict pixels;
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   *pixel=cache_view->image->background_color;
   assert(id < (int) cache_view->number_threads);
-  pixels=GetVirtualPixelsFromNexus(cache_view->image,virtual_pixel_method,x,y,1,
+  pixels=GetVirtualPixelCacheNexus(cache_view->image,virtual_pixel_method,x,y,1,
     1,cache_view->nexus_info[id],exception);
   if (pixels == (const PixelPacket *) NULL)
     return(MagickFalse);
@@ -888,7 +888,7 @@ MagickExport PixelPacket *QueueCacheViewAuthenticPixels(CacheView *cache_view,
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(id < (int) cache_view->number_threads);
   return(QueueAuthenticPixelCacheNexus(cache_view->image,x,y,columns,rows,
     MagickFalse,cache_view->nexus_info[id],exception));
@@ -924,7 +924,7 @@ MagickExport MagickBooleanType SetCacheViewStorageClass(CacheView *cache_view,
   const ClassType storage_class)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -962,7 +962,7 @@ MagickExport MagickBooleanType SetCacheViewVirtualPixelMethod(
   const VirtualPixelMethod virtual_pixel_method)
 {
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   if (cache_view->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       cache_view->image->filename);
@@ -1004,7 +1004,7 @@ MagickExport MagickBooleanType SyncCacheViewAuthenticPixels(
     id = GetOpenMPThreadId();
 
   assert(cache_view != (CacheView *) NULL);
-  assert(cache_view->signature == MagickSignature);
+  assert(cache_view->signature == MagickCoreSignature);
   assert(id < (int) cache_view->number_threads);
   return(SyncAuthenticPixelCacheNexus(cache_view->image,
     cache_view->nexus_info[id],exception));
