@@ -16,13 +16,13 @@
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -193,7 +193,7 @@ static AESInfo *AcquireAESInfo(void)
   aes_info=(AESInfo *) AcquireMagickMemory(sizeof(*aes_info));
   if (aes_info == (AESInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(aes_info,0,sizeof(*aes_info));
+  (void) memset(aes_info,0,sizeof(*aes_info));
   aes_info->blocksize=AESBlocksize;
   aes_info->key=AcquireStringInfo(32);
   aes_info->encipher_key=(unsigned int *) AcquireQuantumMemory(60UL,sizeof(
@@ -205,7 +205,7 @@ static AESInfo *AcquireAESInfo(void)
       (aes_info->decipher_key == (unsigned int *) NULL))
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   aes_info->timestamp=(ssize_t) time(0);
-  aes_info->signature=MagickSignature;
+  aes_info->signature=MagickCoreSignature;
   return(aes_info);
 }
 
@@ -235,7 +235,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(aes_info != (AESInfo *) NULL);
-  assert(aes_info->signature == MagickSignature);
+  assert(aes_info->signature == MagickCoreSignature);
   if (aes_info->decipher_key != (unsigned int *) NULL)
     aes_info->decipher_key=(unsigned int *) RelinquishMagickMemory(
       aes_info->decipher_key);
@@ -244,7 +244,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
       aes_info->encipher_key);
   if (aes_info->key != (StringInfo *) NULL)
     aes_info->key=DestroyStringInfo(aes_info->key);
-  aes_info->signature=(~MagickSignature);
+  aes_info->signature=(~MagickCoreSignature);
   aes_info=(AESInfo *) RelinquishMagickMemory(aes_info);
   return(aes_info);
 }
@@ -483,8 +483,8 @@ static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
     Reset registers.
   */
   alpha=0;
-  (void) ResetMagickMemory(key,0,sizeof(key));
-  (void) ResetMagickMemory(text,0,sizeof(text));
+  (void) memset(key,0,sizeof(key));
+  (void) memset(text,0,sizeof(text));
 }
 
 /*
@@ -604,11 +604,11 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     Generate decipher key and nonce.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
   aes_info=AcquireAESInfo();
@@ -637,9 +637,9 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   UpdateSignature(signature_info,nonce);
   nonce=DestroyStringInfo(nonce);
   FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  (void) memset(input_block,0,sizeof(input_block));
   digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
+  (void) memcpy(input_block,digest,MagickMin(AESBlocksize,
     GetSignatureDigestsize(signature_info))*sizeof(*input_block));
   signature_info=DestroySignatureInfo(signature_info);
   /*
@@ -672,7 +672,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     p=pixels;
     for (x=0; x < (ssize_t) length; x+=AESBlocksize)
     {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+      (void) memmove(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
       IncrementCipherNonce(AESBlocksize,input_block);
       EncipherAESBlock(aes_info,output_block,output_block);
@@ -680,8 +680,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
         p[i]^=output_block[i];
       p+=AESBlocksize;
     }
-    (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
-      sizeof(*output_block));
+    (void) memmove(output_block,input_block,AESBlocksize*sizeof(*output_block));
     EncipherAESBlock(aes_info,output_block,output_block);
     for (i=0; x < (ssize_t) length; x++)
     {
@@ -707,8 +706,8 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   */
   quantum_info=DestroyQuantumInfo(quantum_info);
   aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
+  (void) memset(input_block,0,sizeof(input_block));
+  (void) memset(output_block,0,sizeof(output_block));
   return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
@@ -817,11 +816,11 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     Generate encipher key and nonce.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -852,9 +851,9 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   UpdateSignature(signature_info,nonce);
   nonce=DestroyStringInfo(nonce);
   FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  (void) memset(input_block,0,sizeof(input_block));
   digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
+  (void) memcpy(input_block,digest,MagickMin(AESBlocksize,
     GetSignatureDigestsize(signature_info))*sizeof(*input_block));
   signature=StringInfoToHexString(GetSignatureDigest(signature_info));
   (void) SetImageProperty(image,"cipher:type","AES");
@@ -892,7 +891,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     p=pixels;
     for (x=0; x < (ssize_t) length; x+=AESBlocksize)
     {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+      (void) memmove(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
       IncrementCipherNonce(AESBlocksize,input_block);
       EncipherAESBlock(aes_info,output_block,output_block);
@@ -900,7 +899,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
         p[i]^=output_block[i];
       p+=AESBlocksize;
     }
-    (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+    (void) memmove(output_block,input_block,AESBlocksize*
       sizeof(*output_block));
     EncipherAESBlock(aes_info,output_block,output_block);
     for (i=0; x < (ssize_t) length; x++)
@@ -924,8 +923,8 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   */
   quantum_info=DestroyQuantumInfo(quantum_info);
   aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
+  (void) memset(input_block,0,sizeof(input_block));
+  (void) memset(output_block,0,sizeof(output_block));
   return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
@@ -1011,7 +1010,7 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
   */
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(aes_info != (AESInfo *) NULL);
-  assert(aes_info->signature == MagickSignature);
+  assert(aes_info->signature == MagickCoreSignature);
   assert(key != (StringInfo *) NULL);
   n=4;
   aes_info->rounds=10;
@@ -1030,8 +1029,8 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
     Generate crypt key.
   */
   datum=GetStringInfoDatum(aes_info->key);
-  (void) ResetMagickMemory(datum,0,GetStringInfoLength(aes_info->key));
-  (void) CopyMagickMemory(datum,GetStringInfoDatum(key),MagickMin(
+  (void) memset(datum,0,GetStringInfoLength(aes_info->key));
+  (void) memcpy(datum,GetStringInfoDatum(key),MagickMin(
     GetStringInfoLength(key),GetStringInfoLength(aes_info->key)));
   for (i=0; i < n; i++)
     aes_info->encipher_key[i]=datum[4*i] | (datum[4*i+1] << 8) |
@@ -1065,7 +1064,7 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
     Reset registers.
   */
   datum=GetStringInfoDatum(aes_info->key);
-  (void) ResetMagickMemory(datum,0,GetStringInfoLength(aes_info->key));
+  (void) memset(datum,0,GetStringInfoLength(aes_info->key));
   alpha=0;
   beta=0;
 }
@@ -1107,11 +1106,11 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1120,11 +1119,11 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   const StringInfo *passkey,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passkey;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1163,11 +1162,11 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1176,11 +1175,11 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   const StringInfo *passkey,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passkey;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
