@@ -17,13 +17,13 @@
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -274,7 +274,7 @@ MagickExport void Ascii85Initialize(Image *image)
     image->ascii85=(Ascii85Info *) AcquireMagickMemory(sizeof(*image->ascii85));
   if (image->ascii85 == (Ascii85Info *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(image->ascii85,0,sizeof(*image->ascii85));
+  (void) memset(image->ascii85,0,sizeof(*image->ascii85));
   image->ascii85->line_break=MaxLineExtent << 1;
   image->ascii85->offset=0;
 }
@@ -285,7 +285,7 @@ MagickExport void Ascii85Flush(Image *image)
     *tuple;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->ascii85 != (Ascii85Info *) NULL);
@@ -315,7 +315,7 @@ MagickExport void Ascii85Encode(Image *image,const unsigned char code)
     n;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   assert(image->ascii85 != (Ascii85Info *) NULL);
   image->ascii85->buffer[image->ascii85->offset]=code;
   image->ascii85->offset++;
@@ -453,9 +453,12 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     Allocate buffers.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  exception=(&image->exception);
+  if (image->blob == (BlobInfo *) NULL)
+    ThrowBinaryException(BlobError,"UnableToOpenBlob",image->filename);
   mb_hash=(HuffmanTable **) AcquireQuantumMemory(HashSize,sizeof(*mb_hash));
   mw_hash=(HuffmanTable **) AcquireQuantumMemory(HashSize,sizeof(*mw_hash));
   scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
@@ -463,8 +466,16 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   if ((mb_hash == (HuffmanTable **) NULL) ||
       (mw_hash == (HuffmanTable **) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
+    {
+      if (mb_hash != (HuffmanTable **) NULL)
+        mb_hash=(HuffmanTable **) RelinquishMagickMemory(mb_hash);
+      if (mw_hash != (HuffmanTable **) NULL)
+        mw_hash=(HuffmanTable **) RelinquishMagickMemory(mw_hash);
+      if (scanline != (unsigned char *) NULL)
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   /*
     Initialize Huffman tables.
   */
@@ -492,7 +503,6 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   image->x_resolution=204.0;
   image->y_resolution=196.0;
   image->units=PixelsPerInchResolution;
-  exception=(&image->exception);
   image_view=AcquireAuthenticCacheView(image,exception);
   for (y=0; ((y < (ssize_t) image->rows) && (null_lines < 3)); )
   {
@@ -505,7 +515,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     /*
       Initialize scanline to white.
     */
-    ResetMagickMemory(scanline,0,sizeof(*scanline)*image->columns);
+    memset(scanline,0,sizeof(*scanline)*image->columns);
     /*
       Decode Huffman encoded scanline.
     */
@@ -743,13 +753,13 @@ RestoreMSCWarning \
     Allocate scanline buffer.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(inject_image != (Image *) NULL);
-  assert(inject_image->signature == MagickSignature);
+  assert(inject_image->signature == MagickCoreSignature);
   one=1;
   width=inject_image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
@@ -757,9 +767,9 @@ RestoreMSCWarning \
   scanline=(unsigned char *) AcquireQuantumMemory((size_t) width+1UL,
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
       inject_image->filename);
-  (void) ResetMagickMemory(scanline,0,width*sizeof(*scanline));
+  (void) memset(scanline,0,width*sizeof(*scanline));
   huffman_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
   if (huffman_image == (Image *) NULL)
     {
@@ -925,9 +935,9 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
     number_bits+=code_width; \
     while (number_bits >= 8) \
     { \
-        (void) WriteBlobByte(image,(unsigned char) (accumulator >> 24)); \
-        accumulator=accumulator << 8; \
-        number_bits-=8; \
+      (void) WriteBlobByte(image,(unsigned char) (accumulator >> 24)); \
+      accumulator=accumulator << 8; \
+      number_bits-=8; \
     } \
 }
 
@@ -959,13 +969,14 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
     Allocate string table.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   table=(TableType *) AcquireQuantumMemory(1UL << 12,sizeof(*table));
   if (table == (TableType *) NULL)
-    return(MagickFalse);
+    ThrowBinaryImageException(ResourceLimitWarning,"MemoryAllocationFailed",
+      image->filename);
   /*
     Initialize variables.
   */
@@ -1094,13 +1105,13 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
     Compress pixels with Packbits encoding.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   packbits=(unsigned char *) AcquireQuantumMemory(128UL,sizeof(*packbits));
   if (packbits == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   for (i=(ssize_t) length; i != 0; )
   {
@@ -1247,14 +1258,14 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
     stream;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   compress_packets=(size_t) (1.001*length+12);
   compress_pixels=(unsigned char *) AcquireQuantumMemory(compress_packets,
     sizeof(*compress_pixels));
   if (compress_pixels == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   stream.next_in=pixels;
   stream.avail_in=(unsigned int) length;
@@ -1275,7 +1286,8 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
       compress_packets=(size_t) stream.total_out;
     }
   if (status != Z_OK)
-    ThrowBinaryException(CoderError,"UnableToZipCompressImage",image->filename)
+    ThrowBinaryImageException(CoderError,"UnableToZipCompressImage",
+      image->filename)
   for (i=0; i < (ssize_t) compress_packets; i++)
     (void) WriteBlobByte(image,compress_pixels[i]);
   compress_pixels=(unsigned char *) RelinquishMagickMemory(compress_pixels);
@@ -1286,7 +1298,7 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,
   const size_t magick_unused(length),unsigned char *magick_unused(pixels))
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   (void) ThrowMagickException(&image->exception,GetMagickModule(),
