@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -486,9 +486,10 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_FrameImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,FrameImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,FrameImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -768,9 +769,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RaiseImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,RaiseImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -831,9 +833,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RaiseImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,RaiseImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -905,9 +908,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RaiseImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,RaiseImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
