@@ -16,7 +16,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -800,6 +800,15 @@ static char *GetMagickPropertyLetter(const ImageInfo *image_info,Image *image,
         image->page.width,(double) image->page.height);
       break;
     }
+    case '~':
+    {
+      /*
+        BPG Image compression quality.
+      */
+      (void) FormatLocaleString(value,MaxTextExtent,"%.20g",(double)
+        (100-(image->quality == 0 ? 42 : image->quality))/2);
+      break;
+    }
     case 'Q':
     {
       /*
