@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,8 +15,8 @@
 
   MagickCore deprecated methods.
 */
-#ifndef _MAGICKCORE_DEPRECATE_H
-#define _MAGICKCORE_DEPRECATE_H
+#ifndef MAGICKCORE_DEPRECATE_H
+#define MAGICKCORE_DEPRECATE_H
 
 #include <stdarg.h>
 #include "magick/blob.h"
@@ -82,10 +82,10 @@ typedef struct _ImageAttribute
   char
     *key,
     *value;
-                                                                                
+
   MagickBooleanType
     compression;
-                                                                                
+
   struct _ImageAttribute
     *previous,
     *next;  /* deprecated */
@@ -326,6 +326,7 @@ extern MagickExport void
   AllocateNextImage(const ImageInfo *,Image *) magick_attribute((deprecated)),
   *CloneMemory(void *,const void *,const size_t) magick_attribute((deprecated)),
   ConstituteComponentTerminus(void) magick_attribute((deprecated)),
+  *CropImageToHBITMAP(Image *,const RectangleInfo *,ExceptionInfo *),
   DestroyConstitute(void),
   DestroyImageAttributes(Image *) magick_attribute((deprecated)),
   DestroyImages(Image *) magick_attribute((deprecated)),
@@ -336,7 +337,9 @@ extern MagickExport void
   GetExceptionInfo(ExceptionInfo *),
   *GetMagickRegistry(const ssize_t,RegistryType *,size_t *,ExceptionInfo *)
     magick_attribute((deprecated)),
+  GetTokenToken(const char *,const char **,char *),
   IdentityAffine(AffineMatrix *) magick_attribute((deprecated)),
+  *ImageToHBITMAP(Image *,ExceptionInfo *),
   LiberateMemory(void **) magick_attribute((deprecated)),
   LiberateSemaphoreInfo(SemaphoreInfo **) magick_attribute((deprecated)),
   FormatString(char *,const char *,...)
@@ -373,8 +376,8 @@ static inline double MagickEpsilonReciprocal(const double x)
   return(sign/MagickEpsilon);
 }
 
-static inline Quantum PixelIntensityToQuantum(const Image *restrict image,
-  const PixelPacket *restrict pixel)
+static inline Quantum PixelIntensityToQuantum(
+  const Image *magick_restrict image,const PixelPacket *magick_restrict pixel)
 {
   return(ClampToQuantum(GetPixelIntensity(image,pixel)));
 }
