@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,8 +15,8 @@
 
   MagickCore distributed cache methods.
 */
-#ifndef _MAGICKCORE_DISTRIBUTE_CACHE_H
-#define _MAGICKCORE_DISTRIBUTE_CACHE_H
+#ifndef MAGICKCORE_DISTRIBUTE_CACHE_H
+#define MAGICKCORE_DISTRIBUTE_CACHE_H
 
 #include "magick/exception.h"
 
