@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
 
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -22,6 +22,7 @@
 #include "magick/image.h"
 #include "magick/pixel.h"
 #include "magick/type.h"
+#include "magick/color.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -338,6 +339,18 @@ typedef struct _DrawInfo
   double
     fill_opacity,
     stroke_opacity;
+
+  MagickBooleanType
+    clip_path;
+
+  Image
+    *clipping_mask;
+
+  ComplianceType
+    compliance;
+
+  Image
+    *composite_mask;
 } DrawInfo;
 
 typedef struct _PrimitiveInfo
@@ -356,6 +369,9 @@ typedef struct _PrimitiveInfo
 
   char
     *text;
+
+  MagickBooleanType
+    closed_subpath;
 } PrimitiveInfo;
 
 typedef struct _TypeMetric
