@@ -22,7 +22,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -305,9 +305,10 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_DuplexTransferImageViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -590,9 +591,10 @@ MagickExport MagickBooleanType GetImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_GetImageViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -951,10 +953,11 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_SetImageViewIterator)
+        #pragma omp atomic
 #endif
+        progress++;
         proceed=SetImageProgress(destination_image,destination->description,
-          progress++,destination->extent.height);
+          progress,destination->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1130,9 +1133,10 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_TransferImageViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -1253,9 +1257,10 @@ MagickExport MagickBooleanType UpdateImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_UpdateImageViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
