@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -864,9 +864,6 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=tile_height,
               image->rows);
             if (proceed == MagickFalse)
@@ -940,10 +937,10 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_IntegralRotateImage)
+            #pragma omp atomic
 #endif
-            proceed=SetImageProgress(image,RotateImageTag,progress++,
-              image->rows);
+            progress++;
+            proceed=SetImageProgress(image,RotateImageTag,progress,image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
@@ -1064,9 +1061,6 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             MagickBooleanType
               proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_IntegralRotateImage)
-#endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=tile_height,
               image->rows);
             if (proceed == MagickFalse)
@@ -1303,9 +1297,10 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_XShearImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,XShearImageTag,progress++,height);
+        progress++;
+        proceed=SetImageProgress(image,XShearImageTag,progress,height);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1524,9 +1519,10 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_YShearImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,YShearImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,YShearImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
