@@ -16,7 +16,7 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -132,6 +132,8 @@ MagickExport SignatureInfo *AcquireSignatureInfo(void)
     SignatureBlocksize,sizeof(*signature_info->accumulator));
   if (signature_info->accumulator == (unsigned int *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memset(signature_info->accumulator,0,SignatureBlocksize*
+    sizeof(*signature_info->accumulator));
   lsb_first=1;
   signature_info->lsb_first=(int) (*(char *) &lsb_first) == 1 ? MagickTrue :
     MagickFalse;
@@ -569,33 +571,16 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
 %    o signature_info: the address of a structure of type SignatureInfo.
 %
 */
-
-static inline unsigned int Ch(unsigned int x,unsigned int y,unsigned int z)
-{
-  return((x & y) ^ (~x & z));
-}
-
-static inline unsigned int Maj(unsigned int x,unsigned int y,unsigned int z)
-{
-  return((x & y) ^ (x & z) ^ (y & z));
-}
-
-static inline unsigned int Trunc32(unsigned int x)
-{
-  return((unsigned int) (x & 0xffffffffU));
-}
-
-static unsigned int RotateRight(unsigned int x,unsigned int n)
-{
-  return(Trunc32((x >> n) | (x << (32-n))));
-}
-
 static void TransformSignature(SignatureInfo *signature_info)
 {
+#define Ch(x,y,z)  (((x) & (y)) ^ (~(x) & (z)))
+#define Maj(x,y,z)  (((x) & (y)) ^ ((x) & (z)) ^ ((y) & (z)))
+#define RotateRight(x,n)  (Trunc32(((x) >> n) | ((x) << (32-n))))
 #define Sigma0(x)  (RotateRight(x,7) ^ RotateRight(x,18) ^ Trunc32((x) >> 3))
 #define Sigma1(x)  (RotateRight(x,17) ^ RotateRight(x,19) ^ Trunc32((x) >> 10))
 #define Suma0(x)  (RotateRight(x,2) ^ RotateRight(x,13) ^ RotateRight(x,22))
 #define Suma1(x)  (RotateRight(x,6) ^ RotateRight(x,11) ^ RotateRight(x,25))
+#define Trunc32(x)  ((unsigned int) ((x) & 0xffffffffU))
 
   register ssize_t
     i;
@@ -606,7 +591,7 @@ static void TransformSignature(SignatureInfo *signature_info)
   ssize_t
     j;
 
-  static unsigned int
+  static const unsigned int
     K[64] =
     {
       0x428a2f98U, 0x71374491U, 0xb5c0fbcfU, 0xe9b5dba5U, 0x3956c25bU,
