@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -37,6 +37,9 @@ typedef struct _StringInfo
   size_t
     length,
     signature;
+
+  char
+    *name;
 } StringInfo;
 
 extern MagickExport char
@@ -56,6 +59,7 @@ extern MagickExport char
   **StringToList(const char *);
 
 extern MagickExport const char
+  *GetStringInfoName(const StringInfo *),
   *GetStringInfoPath(const StringInfo *);
 
 extern MagickExport double
@@ -67,8 +71,8 @@ extern MagickExport int
 
 extern MagickExport MagickBooleanType
   ConcatenateString(char **,const char *),
-  IsStringTrue(const char *),
-  IsStringNotFalse(const char *),
+  IsStringTrue(const char *) magick_attribute((__pure__)),
+  IsStringNotFalse(const char *) magick_attribute((__pure__)),
   SubstituteString(char **,const char *,const char *);
 
 extern MagickExport size_t
@@ -103,6 +107,7 @@ extern MagickExport void
   SetStringInfo(StringInfo *,const StringInfo *),
   SetStringInfoDatum(StringInfo *,const unsigned char *),
   SetStringInfoLength(StringInfo *,const size_t),
+  SetStringInfoName(StringInfo *,const char *),
   SetStringInfoPath(StringInfo *,const char *),
   StripString(char *);
 
