@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -33,9 +33,10 @@ extern MagickExport int
     const char *,const char *,const char *,const char,char *,int *,char *);
 
 extern MagickExport MagickBooleanType
-  GlobExpression(const char *,const char *,const MagickBooleanType),
-  IsGlob(const char *),
-  IsMagickTrue(const char *);
+  GlobExpression(const char *,const char *,const MagickBooleanType)
+    magick_attribute((__pure__)),
+  IsGlob(const char *) magick_attribute((__pure__)),
+  IsMagickTrue(const char *) magick_attribute((__pure__));
 
 extern MagickExport TokenInfo
   *AcquireTokenInfo(void),
