@@ -22,7 +22,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -400,9 +400,10 @@ WandExport MagickBooleanType DuplexTransferWandViewIterator(WandView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickWand_DuplexTransferWandViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -616,9 +617,10 @@ WandExport MagickBooleanType GetWandViewIterator(WandView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickWand_GetWandViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -1033,10 +1035,11 @@ WandExport MagickBooleanType SetWandViewIterator(WandView *destination,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickWand_SetWandViewIterator)
+        #pragma omp atomic
 #endif
+        progress++;
         proceed=SetImageProgress(destination_image,destination->description,
-          progress++,destination->extent.height);
+          progress,destination->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1250,9 +1253,10 @@ WandExport MagickBooleanType TransferWandViewIterator(WandView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickWand_TransferWandViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -1393,9 +1397,10 @@ WandExport MagickBooleanType UpdateWandViewIterator(WandView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickWand_UpdateWandViewIterator)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(source_image,source->description,progress++,
+        progress++;
+        proceed=SetImageProgress(source_image,source->description,progress,
           source->extent.height);
         if (proceed == MagickFalse)
           status=MagickFalse;
